#include <stdio.h>
#include <string.h>

#include "petopt.h"

static int parse_option();

int verbose = 0;
int debug = 0;
char *filename = NULL;

/*
** Fields are:
**   Short option character
**   Type & Flags
**   Long option name
**   Variable
**   Help text
*/
PETOPTS pov[] =
{
    { 'h', 0, 			"HeLP",		 NULL,      "Print help" },
    { 'v', POF_BOOL,		"Verbose",	 &verbose,  "Verbose mode" },
    { 'd', POF_OPT|POF_INT,	"DeBuG",	 &debug,    "Debug mode" },
    { 'f', POF_STR,		"File",		 &filename, "Input file" },
    { 'q', POF_OPT|POF_STR,	"Que",		 NULL,      NULL },
    { 'x', POF_INT, 		NULL,		 NULL,      "Extended mode" },
    { 1,   0,			"Foo-SiGNature", NULL,      "Signature" },
    { 'A', 0, 			"ENable-ALpha",  NULL,      "Alpha mode" },
    { 'B', 0,			"ENable-Beta",	 NULL,      "Beta mode" },
    { -1,  0, 			NULL,		 NULL,      NULL }
};





/*
** Valid return values:
**   <0. ...... Some error occured
**    0 ....... Option parsed and stored ok
**    1 ....... Option parsed, use builtin store function
*/
static int
parse_option(PETOPT *pp,
	     PETOPTS *pov,
	     const char *arg)
{
    switch (pov->s)
    {
      case 1:
	printf("Option: Signature, Arg: %s\n", arg ? arg : "<null>");
	break;
	
      case 'A':
	printf("Option: Alpha, Arg: %s\n", arg ? arg : "<null>");
	break;
	
      case 'B':
	printf("Option: Beta, Arg: %s\n", arg ? arg : "<null>");
	break;
	
      case 'h':
	petopt_print_usage(pp, stdout);
	exit(0);
	
      case 'v':
	printf("Option: Verbose, Arg: %s\n", arg ? arg : "<null>");
	return 1;
	
      case 'q':
	printf("Option: Que, Arg: %s\n", arg ? arg : "<null>");
	break;
	
      case 'x':
	printf("Option: -x, Arg: %s\n", arg ? arg : "<null>");
	return 1;
	
      default:
	return 1;
    }

    return 0;
}


int
main(int argc,
     char *argv[])
{
    int err, i;
    PETOPT *pop;
    

    err = petopt_setup(&pop, 0,
		       argc, argv,
		       pov,
		       parse_option, NULL);
    if (err)
    {
	if (err > 0)
	    fprintf(stderr, "petopt_setup: %s\n", strerror(err));
	exit(1);
    }
    
    err = petopt_parse(pop, &argc, &argv);
    if (err)
    {
	if (err > 0)
	    fprintf(stderr, "petopt_parse_all: %s\n", strerror(err));
	exit(1);
    }
    
    if (argc > 0)
    {
	printf("Extra arguments (%d):\n", argc);
	for (i = 0; i < argc; i++)
	    printf("  %2d = %s\n", i, argv[i]);
    }

    err = petopt_cleanup(pop);
    if (err)
    {
	if (err > 0)
	    fprintf(stderr, "petopt_cleanup: %s\n", strerror(err));
	exit(1);
    }

    printf("Debug: %d\n", debug);
    printf("Verbose: %d\n", verbose);
    printf("Filename: %s\n", filename ? filename : "<null>");
    exit(0);
}
